@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2018 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2011-2019 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -9,8 +9,7 @@ import sys
 import struct
 
 from xpra.os_util import bytestostr, hexstr
-from xpra.util import iround, envbool, envint, csv
-from xpra.gtk_common.gtk_util import get_xwindow
+from xpra.util import iround, envbool, envint, csv, repr_ellipsized
 from xpra.os_util import is_unity, is_gnome, is_kde, is_Fedora, is_X11, is_Wayland
 from xpra.log import Logger
 
@@ -19,19 +18,37 @@ eventlog = Logger("posix", "events")
 screenlog = Logger("posix", "screen")
 dbuslog = Logger("posix", "dbus")
 traylog = Logger("posix", "tray")
-menulog = Logger("posix", "menu")
 mouselog = Logger("posix", "mouse")
 xinputlog = Logger("posix", "xinput")
 
-X11WindowBindings = None
-X11XI2Bindings = None
-if is_X11():
-    try:
-        from xpra.x11.bindings.window_bindings import X11WindowBindings #@UnresolvedImport
-        from xpra.x11.bindings.xi2_bindings import X11XI2Bindings       #@UnresolvedImport
-    except Exception as e:
-        log.error("no X11 bindings", exc_info=True)
-        del e
+_X11Window = False
+def X11WindowBindings():
+    global _X11Window
+    if _X11Window is False:
+        _X11Window = None
+        if is_X11():
+            try:
+                from xpra.x11.bindings.window_bindings import X11WindowBindings as _X11WindowBindings #@UnresolvedImport
+                _X11Window = _X11WindowBindings()
+            except Exception as e:
+                log("X11WindowBindings()", exc_info=True)
+                log.error("Error: no X11 bindings")
+                log.error(" %s", e)
+    return _X11Window
+
+X11XI2 = False
+def X11XI2Bindings():
+    global X11XI2
+    if X11XI2 is False:
+        X11XI2 = None
+        if is_X11():
+            try:
+                from xpra.x11.bindings.xi2_bindings import X11XI2Bindings as _X11XI2Bindings       #@UnresolvedImport
+                X11XI2 = _X11XI2Bindings()
+            except Exception:
+                log.error("no XI2 bindings", exc_info=True)
+    return X11XI2
+
 
 device_bell = None
 GTK_MENUS = envbool("XPRA_GTK_MENUS", False)
@@ -42,6 +59,10 @@ XINPUT_WHEEL_DIV = envint("XPRA_XINPUT_WHEEL_DIV", 15)
 DBUS_SCREENSAVER = envbool("XPRA_DBUS_SCREENSAVER", False)
 
 
+def get_xwindow(win):
+    from xpra.gtk_common.gtk_util import get_xwindow as _get_xwindow
+    return _get_xwindow(win)
+
 def gl_check():
     if not is_X11() and is_Wayland():
         return "disabled under wayland with GTK3 (buggy)"
@@ -53,11 +74,13 @@ def get_native_system_tray_classes():
 
 def get_wm_name():
     wm_name = os.environ.get("XDG_CURRENT_DESKTOP", "")
-    if is_X11():
+    if os.environ.get("XDG_SESSION_TYPE")=="wayland" or os.environ.get("GDK_BACKEND")=="wayland":
+        wm_name = "wayland"
+    elif is_X11():
         try:
             wm_check = _get_X11_root_property("_NET_SUPPORTING_WM_CHECK", "WINDOW")
             if wm_check:
-                xid = struct.unpack(b"=I", wm_check)[0]
+                xid = struct.unpack(b"@L", wm_check)[0]
                 traylog("_NET_SUPPORTING_WM_CHECK window=%#x", xid)
                 wm_name = _get_X11_window_property(xid, "_NET_WM_NAME", "UTF8_STRING")
                 traylog("_NET_WM_NAME=%s", wm_name)
@@ -69,6 +92,11 @@ def get_wm_name():
             traylog.error(" %s", e)
     return wm_name
 
+def get_clipboard_native_class():
+    if is_Wayland():
+        return None
+    return "xpra.x11.gtk_x11.clipboard.X11Clipboard"
+
 def get_native_tray_classes():
     #could restrict to only DEs that have a broken system tray like "GNOME Shell"?
     c = []
@@ -78,13 +106,6 @@ def get_native_tray_classes():
             c.append(AppindicatorTray)
         except (ImportError, ValueError):
             traylog("cannot load appindicator tray", exc_info=True)
-    if has_gtk_menu_support():  #and wm_name=="GNOME Shell":
-        try:
-            from xpra.platform.xposix.gtkmenu_tray import GTKMenuTray
-            traylog("using GTKMenuTray for '%s' window manager", get_wm_name() or "unknown")
-            c.append(GTKMenuTray)
-        except Exception as e:
-            traylog("cannot load gtk menu tray: %s", e)
     return c
 
 
@@ -114,9 +135,8 @@ def _get_X11_window_property(xid, name, req_type):
         from xpra.gtk_common.error import xsync
         from xpra.x11.bindings.window_bindings import PropertyError #@UnresolvedImport
         try:
-            X11Window = X11WindowBindings()
             with xsync:
-                prop = X11Window.XGetWindowProperty(xid, name, req_type)
+                prop = X11WindowBindings().XGetWindowProperty(xid, name, req_type)
             log("_get_X11_window_property(%#x, %s, %s)=%s, len=%s", xid, name, req_type, type(prop), len(prop or []))
             return prop
         except PropertyError as e:
@@ -127,8 +147,7 @@ def _get_X11_window_property(xid, name, req_type):
     return None
 def _get_X11_root_property(name, req_type):
     try:
-        X11Window = X11WindowBindings()
-        root_xid = X11Window.getDefaultRootWindow()
+        root_xid = X11WindowBindings().getDefaultRootWindow()
         return _get_X11_window_property(root_xid, name, req_type)
     except Exception as e:
         log("_get_X11_root_property(%s, %s)", name, req_type, exc_info=True)
@@ -137,77 +156,26 @@ def _get_X11_root_property(name, req_type):
     return None
 
 
-def _set_gtk_x11_window_menu(add, wid, window, menus, application_action_callback=None, window_action_callback=None):
-    from xpra.x11.dbus.menu import setup_dbus_window_menu
-    from xpra.x11.gtk_x11.prop import prop_set, prop_del
-    window_props = setup_dbus_window_menu(add, wid, menus, application_action_callback, window_action_callback)
-    #window_props may contains X11 window properties we have to clear or set
-    if not window_props:
-        return
-    if not window:
-        #window has already been closed
-        #(but we still want to call setup_dbus_window_menu above to ensure we clear things up!)
-        return
-    menulog("will set/remove the following window properties for wid=%i: %s", wid, window_props)
-    try:
-        from xpra.gtk_common.error import xsync
-        with xsync:
-            for k,v in window_props.items():
-                if v is None:
-                    prop_del(window, k)
-                else:
-                    vtype, value = v
-                    prop_set(window, k, vtype, value)
-    except Exception as e:
-        menulog.error("Error setting menu window properties:")
-        menulog.error(" %s", e)
-
-
-_has_gtk_menu_support = None
-def has_gtk_menu_support():
-    global _has_gtk_menu_support
-    if not GTK_MENUS:
-        _has_gtk_menu_support = False
-    if _has_gtk_menu_support is not None:
-        return _has_gtk_menu_support
-    try:
-        from xpra.gtk_common.gtk_util import get_default_root_window
-        from xpra.x11.dbus.menu import has_gtk_menu_support as __has_gtk_menu_support
-        root = get_default_root_window()
-        _has_gtk_menu_support = __has_gtk_menu_support(root)
-        menulog("has_gtk_menu_support(%s)=%s", root, _has_gtk_menu_support)
-    except Exception as e:
-        menulog("cannot enable gtk-x11 menu support: %s", e)
-        _has_gtk_menu_support = False
-    return _has_gtk_menu_support
-
-def get_menu_support_function():
-    if has_gtk_menu_support():
-        return _set_gtk_x11_window_menu
-    return None
-
-
 def _get_xsettings():
-    from xpra.gtk_common.error import xsync
-    try:
-        X11Window = X11WindowBindings()
-        with xsync:
-            selection = "_XSETTINGS_S0"
-            owner = X11Window.XGetSelectionOwner(selection)
-            if not owner:
-                return None
-            XSETTINGS = "_XSETTINGS_SETTINGS"
-            data = X11Window.XGetWindowProperty(owner, XSETTINGS, XSETTINGS)
-            if not data:
-                return None
-            from xpra.x11.xsettings_prop import get_settings
-            return get_settings(X11Window.get_display_name(), data)
-    except Exception as e:
-        log("_get_xsettings()", exc_info=True)
+    from xpra.gtk_common.error import xlog
+    X11Window = X11WindowBindings()
+    with xlog:
+        selection = "_XSETTINGS_S0"
+        owner = X11Window.XGetSelectionOwner(selection)
+        if not owner:
+            return None
+        XSETTINGS = "_XSETTINGS_SETTINGS"
+        data = X11Window.XGetWindowProperty(owner, XSETTINGS, XSETTINGS)
+        if not data:
+            return None
+        from xpra.x11.xsettings_prop import get_settings
+        return get_settings(X11Window.get_display_name(), data)
     return None
 
 def _get_xsettings_dict():
     d = {}
+    if is_Wayland():
+        return d
     v = _get_xsettings()
     if v:
         _, values = v
@@ -236,19 +204,16 @@ def _get_xsettings_dpi():
 
 def _get_randr_dpi():
     if RANDR_DPI and not is_Wayland():
-        try:
-            from xpra.gtk_common.error import xsync
-            from xpra.x11.bindings.randr_bindings import RandRBindings  #@UnresolvedImport
-            with xsync:
-                randr_bindings = RandRBindings()
-                wmm, hmm = randr_bindings.get_screen_size_mm()
-                w, h =  randr_bindings.get_screen_size()
-                dpix = iround(w * 25.4 / wmm)
-                dpiy = iround(h * 25.4 / hmm)
-                screenlog("xdpi=%s, ydpi=%s - size-mm=%ix%i, size=%ix%i", dpix, dpiy, wmm, hmm, w, h)
-                return dpix, dpiy
-        except Exception as e:
-            screenlog.warn("failed to get dpi: %s", e)
+        from xpra.gtk_common.error import xlog
+        from xpra.x11.bindings.randr_bindings import RandRBindings  #@UnresolvedImport
+        with xlog:
+            randr_bindings = RandRBindings()
+            wmm, hmm = randr_bindings.get_screen_size_mm()
+            w, h =  randr_bindings.get_screen_size()
+            dpix = iround(w * 25.4 / wmm)
+            dpiy = iround(h * 25.4 / hmm)
+            screenlog("xdpi=%s, ydpi=%s - size-mm=%ix%i, size=%ix%i", dpix, dpiy, wmm, hmm, w, h)
+            return dpix, dpiy
     return -1, -1
 
 def get_xdpi():
@@ -310,11 +275,11 @@ def get_antialias_info():
             return {"hintnone"      : 0,
                     "hintslight"    : 1000,
                     "hintmedium"    : 1600,
-                    "hintfull"      : 2200}.get(value)
+                    "hintfull"      : 2200}.get(bytestostr(value))
         for prop_name, name, convert in (
-                                         ("Xft/HintStyle",  "hintstyle",    str),
+                                         ("Xft/HintStyle",  "hintstyle",    bytestostr),
                                          ("Xft/HintStyle",  "contrast",     get_contrast),
-                                         ("Xft/RGBA",       "orientation",  lambda x : str(x).upper())
+                                         ("Xft/RGBA",       "orientation",  lambda x : bytestostr(x).upper())
                                          ):
             if prop_name in d:
                 value_type, value = d.get(prop_name)
@@ -335,7 +300,7 @@ def get_current_desktop():
         try:
             d = _get_X11_root_property("_NET_CURRENT_DESKTOP", "CARDINAL")
             if d:
-                v = struct.unpack(b"=I", d)[0]
+                v = struct.unpack(b"@L", d)[0]
         except Exception as e:
             log.warn("failed to get current desktop: %s", e)
         log("get_current_desktop() %s=%s", hexstr(d or ""), v)
@@ -350,17 +315,20 @@ def get_workarea():
             workarea = _get_X11_root_property("_NET_WORKAREA", "CARDINAL")
             if not workarea:
                 return None
-            screenlog("get_workarea()=%s, len=%s", type(workarea), len(workarea))
+            screenlog("get_workarea() _NET_WORKAREA=%s (%s), len=%s",
+                      repr_ellipsized(workarea), type(workarea), len(workarea))
             #workarea comes as a list of 4 CARDINAL dimensions (x,y,w,h), one for each desktop
-            if len(workarea)<(d+1)*4*4:
+            sizeof_long = struct.calcsize(b"@L")
+            if len(workarea)<(d+1)*4*sizeof_long:
                 screenlog.warn("get_workarea() invalid _NET_WORKAREA value")
             else:
-                cur_workarea = workarea[d*4*4:(d+1)*4*4]
-                v = struct.unpack(b"=IIII", cur_workarea)
+                cur_workarea = workarea[d*4*sizeof_long:(d+1)*4*sizeof_long]
+                v = struct.unpack(b"@LLLL", cur_workarea)
                 screenlog("get_workarea() %s=%s", hexstr(cur_workarea), v)
                 return v
         except Exception as e:
-            screenlog.warn("failed to get workarea: %s", e)
+            screenlog("get_workarea()", exc_info=True)
+            screenlog.warn("Warning: failed to query workarea: %s", e)
     return None
 
 
@@ -371,7 +339,7 @@ def get_number_of_desktops():
         try:
             d = _get_X11_root_property("_NET_NUMBER_OF_DESKTOPS", "CARDINAL")
             if d:
-                v = struct.unpack(b"=I", d)[0]
+                v = struct.unpack(b"@L", d)[0]
         except Exception as e:
             screenlog.warn("failed to get number of desktop: %s", e)
         v = max(1, v)
@@ -417,11 +385,10 @@ def _get_xresources():
             from xpra.x11.gtk_x11.prop import prop_get
             from xpra.gtk_common.gtk_util import get_default_root_window
             root = get_default_root_window()
-            v = prop_get(root, "RESOURCE_MANAGER", "latin1", ignore_errors=True)
-            log("RESOURCE_MANAGER=%s", v)
-            if v is None:
+            value = prop_get(root, "RESOURCE_MANAGER", "latin1", ignore_errors=True)
+            log("RESOURCE_MANAGER=%s", value)
+            if value is None:
                 return None
-            value = v.decode("utf-8")
             #parse the resources into a dict:
             values={}
             options = value.split("\n")
@@ -486,8 +453,8 @@ def system_bell(window, device, percent, _pitch, _duration, bell_class, bell_id,
             device_bell = X11KeyboardBindings().device_bell
         device_bell(get_xwindow(window), device, bell_class, bell_id, percent, bell_name)
     try:
-        from xpra.gtk_common.error import xsync
-        with xsync:
+        from xpra.gtk_common.error import xlog
+        with xlog:
             x11_bell()
         return  True
     except XError as e:
@@ -583,11 +550,11 @@ class XI2_Window(object):
         window.connect("configure-event", self.configured)
         self.configured()
         #replace event handlers with XI2 version:
-        self.do_motion_notify_event = window.do_motion_notify_event
-        window.do_motion_notify_event = self.noop
-        window.do_button_press_event = self.noop
-        window.do_button_release_event = self.noop
-        window.do_scroll_event = self.noop
+        self._do_motion_notify_event = window._do_motion_notify_event
+        window._do_motion_notify_event = self.noop
+        window._do_button_press_event = self.noop
+        window._do_button_release_event = self.noop
+        window._do_scroll_event = self.noop
         window.connect("destroy", self.cleanup)
 
     def noop(self, *args):
@@ -600,10 +567,10 @@ class XI2_Window(object):
         self.window = None
 
     def configured(self, *_args):
-        from xpra.gtk_common.error import xsync
-        with xsync:
+        from xpra.gtk_common.error import xlog
+        with xlog:
             self.windows = self.get_parent_windows(self.xid)
-        for window in self.windows:
+        for window in (self.windows or ()):
             self.XI2.connect(window, "XI_Motion", self.do_xi_motion)
             self.XI2.connect(window, "XI_ButtonPress", self.do_xi_button)
             self.XI2.connect(window, "XI_ButtonRelease", self.do_xi_button)
@@ -654,7 +621,7 @@ class XI2_Window(object):
         if window.moveresize_event:
             xinputlog("do_xi_motion(%s, %s) handling as a moveresize event on window %s", event, device, window)
             window.motion_moveresize(event)
-            self.do_motion_notify_event(event)
+            self._do_motion_notify_event(event)
             return
         client = window._client
         if client.readonly:
@@ -732,7 +699,7 @@ class XI2_Window(object):
         args = [event.device]
         for x in ("x", "y", "x_root", "y_root"):
             args.append(intscaled(getattr(event, x)))
-        for v in [event.valuators, raw_valuators]:
+        for v in (event.valuators, raw_valuators):
             args.append(dictscaled(v))
         return args
 
@@ -769,7 +736,8 @@ class ClientExtras(object):
             from xpra.x11.gtk_x11.gdk_bindings import init_x11_filter  #@UnresolvedImport, @UnusedImport
             self.x11_filter = init_x11_filter()
             log("x11_filter=%s", self.x11_filter)
-        except Exception:
+        except Exception as e:
+            log("init_x11_filter()", exc_info=True)
             log.error("Error: failed to initialize X11 GDK filter:")
             log.error(" %s", e)
             self.x11_filter = None
