@@ -12,7 +12,7 @@ import math
 from collections import deque, namedtuple
 
 from xpra.version_util import XPRA_VERSION
-from xpra.util import updict, rindex, envbool, envint
+from xpra.util import updict, rindex, envbool, envint, typedict
 from xpra.os_util import memoryview_to_bytes, strtobytes, bytestostr, monotonic_time
 from xpra.server import server_features
 from xpra.gtk_common.gobject_util import one_arg_signal
@@ -24,14 +24,13 @@ from xpra.x11.common import Unmanageable, MAX_WINDOW_SIZE
 from xpra.x11.gtk_x11.prop import prop_set
 from xpra.x11.gtk_x11.tray import get_tray_window, SystemTray
 from xpra.x11.gtk_x11.gdk_bindings import (
-   add_event_receiver, cleanup_all_event_receivers,
+   add_event_receiver,
    get_children,
-   init_x11_filter, cleanup_x11_filter,
    )
 from xpra.x11.bindings.window_bindings import X11WindowBindings #@UnresolvedImport
 from xpra.x11.bindings.keyboard_bindings import X11KeyboardBindings #@UnresolvedImport
 from xpra.x11.x11_server_base import X11ServerBase
-from xpra.gtk_common.error import xsync, xswallow
+from xpra.gtk_common.error import xsync, xswallow, xlog, XError
 from xpra.gtk_common.gobject_compat import import_gtk, import_gdk, import_glib, import_gobject, is_gtk3
 from xpra.log import Logger
 
@@ -44,7 +43,6 @@ traylog = Logger("server", "tray")
 workspacelog = Logger("x11", "workspace")
 metadatalog = Logger("x11", "metadata")
 framelog = Logger("x11", "frame")
-menulog  = Logger("x11", "menu")
 eventlog = Logger("x11", "events")
 mouselog = Logger("x11", "mouse")
 
@@ -60,6 +58,7 @@ REPARENT_ROOT = envbool("XPRA_REPARENT_ROOT", True)
 CONFIGURE_DAMAGE_RATE = envint("XPRA_CONFIGURE_DAMAGE_RATE", 250)
 SHARING_SYNC_SIZE = envbool("XPRA_SHARING_SYNC_SIZE", True)
 CLAMP_WINDOW_TO_ROOT = envbool("XPRA_CLAMP_WINDOW_TO_ROOT", False)
+ALWAYS_RAISE_WINDOW = envbool("XPRA_ALWAYS_RAISE_WINDOW", False)
 
 WINDOW_SIGNALS = os.environ.get("XPRA_WINDOW_SIGNALS", "SIGINT,SIGTERM,SIGQUIT,SIGCONT,SIGUSR1,SIGUSR2").split(",")
 
@@ -197,17 +196,17 @@ class XpraServer(gobject.GObject, X11ServerBase):
         }
 
     def __init__(self, clobber):
-        self.clobber = clobber
         self.root_overlay = None
         self.repaint_root_overlay_timer = None
         self.configure_damage_timers = {}
         self._tray = None
+        self._has_grab = 0
+        self._has_focus = 0
         self._wm = None
         self.last_raised = None
         self.system_tray = False
-        self.global_menus = False
         gobject.GObject.__init__(self)
-        X11ServerBase.__init__(self)
+        X11ServerBase.__init__(self, clobber)
         self.session_type = "seamless"
 
     def init(self, opts):
@@ -216,16 +215,28 @@ class XpraServer(gobject.GObject, X11ServerBase):
         self.system_tray = opts.system_tray
         X11ServerBase.init(self, opts)
         self.fake_xinerama = opts.fake_xinerama
-        #rpc_handlers are part of DBUS_RPC_Server, which is optional
-        rpc_handlers = getattr(self, "rpc_handlers", None)
-        if rpc_handlers:
-            self.global_menus = int(opts.global_menus)
-            if self.global_menus:
-                self.rpc_handlers["menu"] = self._handle_menu_rpc
         def log_server_event(_, event):
             eventlog("server-event: %s", event)
         self.connect("server-event", log_server_event)
 
+    def server_init(self):
+        X11ServerBase.server_init(self)
+        if not self.clobber and self.randr:
+            from xpra.x11.vfb_util import set_initial_resolution, DEFAULT_VFB_RESOLUTION
+            with xlog:
+                set_initial_resolution(DEFAULT_VFB_RESOLUTION)
+
+    def server_ready(self):
+        if not X11Window.displayHasXComposite():
+            log.error("Xpra 'start' subcommand runs as a compositing manager")
+            log.error(" it cannot use a display which lacks the XComposite extension!")
+            return False
+        #check for an existing window manager:
+        from xpra.x11.gtk_x11.wm_check import wm_check
+        if not wm_check(self.wm_name, self.clobber):
+            return False
+        return True
+
     def setup(self):
         X11ServerBase.setup(self)
         if self.system_tray:
@@ -233,10 +244,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
 
     def x11_init(self):
         X11ServerBase.x11_init(self)
-        assert init_x11_filter() is True
 
-        self._has_grab = 0
-        self._has_focus = 0
         self._focus_history = deque(maxlen=100)
         # Do this before creating the Wm object, to avoid clobbering its
         # selecting SubstructureRedirect.
@@ -286,9 +294,6 @@ class XpraServer(gobject.GObject, X11ServerBase):
                 X11Window.XCompositeReleaseOverlayWindow(self.root_overlay)
             self.root_overlay = None
         self.cancel_all_configure_damage()
-        X11ServerBase.do_cleanup(self)
-        cleanup_x11_filter()
-        cleanup_all_event_receivers()
         if self._wm:
             self._wm.cleanup()
             self._wm = None
@@ -297,6 +302,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
             #but at this point in the cleanup, we probably won't, so force set it:
             self._has_grab = 0
             self.X11_ungrab()
+        X11ServerBase.do_cleanup(self)
 
 
     def last_client_exited(self):
@@ -320,9 +326,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
 
     def init_packet_handlers(self):
         X11ServerBase.init_packet_handlers(self)
-        self._authenticated_ui_packet_handlers.update({
-            "window-signal"         : self._process_window_signal,
-            })
+        self.add_packet_handler("window-signal", self._process_window_signal)
 
 
     def get_server_mode(self):
@@ -552,6 +556,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
         window.managed_connect("ungrab", self._window_ungrab)
         window.managed_connect("bell", self._bell_signaled)
         window.managed_connect("motion", self._motion_signaled)
+        window.managed_connect("x11-property-changed", self._x11_property_changed)
         if not window.is_tray():
             window.managed_connect("raised", self._raised_window)
             window.managed_connect("initiate-moveresize", self._initiate_moveresize)
@@ -564,6 +569,17 @@ class XpraServer(gobject.GObject, X11ServerBase):
                 #but we don't really care, it will get cleaned up soon enough
         return wid
 
+
+    def _x11_property_changed(self, window, event):
+        #name, dtype, dformat, value = event
+        metadata = {"x11-property" : event}
+        wid = self._window_to_id[window]
+        for ss in self._server_sources.values():
+            ms = getattr(ss, "metadata_supported", ())
+            if "x11-property" in ms:
+                ss.send("window-metadata", wid, metadata)
+
+
     def _add_new_window(self, window):
         self._add_new_window_common(window)
         x, y, w, h = window.get_property("geometry")
@@ -831,14 +847,15 @@ class XpraServer(gobject.GObject, X11ServerBase):
         assert proto in self._server_sources
         if not new_window_state:
             return []
+        nws = typedict(new_window_state)
         metadatalog("set_window_state%s", (wid, window, new_window_state))
         changes = []
         if "frame" in new_window_state:
             #the size of the window frame may have changed
-            frame = new_window_state.get("frame") or (0, 0, 0, 0)
+            frame = nws.intlistget("frame", (0, 0, 0, 0))
             window.set_property("frame", frame)
         #boolean: but not a wm_state and renamed in the model... (iconic vs inconified!)
-        iconified = new_window_state.get("iconified")
+        iconified = nws.boolget("iconified")
         if iconified is not None:
             if window.is_OR():
                 log("ignoring iconified=%s on OR window %s", iconified, window)
@@ -853,14 +870,15 @@ class XpraServer(gobject.GObject, X11ServerBase):
             "sticky", "shaded",
             "skip-pager", "skip-taskbar", "focused",
             ):
-            if k in new_window_state:
-                #metadatalog.info("window.get_property=%s", window.get_property)
-                new_state = bool(new_window_state.get(k, False))
-                cur_state = bool(window.get_property(k))
-                #metadatalog.info("set window state for '%s': current state=%s, new state=%s", k, cur_state, new_state)
-                if cur_state!=new_state:
-                    window.update_wm_state(k, new_state)
-                    changes.append(k)
+            #metadatalog.info("window.get_property=%s", window.get_property)
+            new_state = nws.boolget(k, None)
+            if new_state is None:
+                continue
+            cur_state = bool(window.get_property(k))
+            #metadatalog.info("set window state for '%s': current state=%s, new state=%s", k, cur_state, new_state)
+            if cur_state!=new_state:
+                window.update_wm_state(k, new_state)
+                changes.append(k)
         metadatalog("set_window_state: changes=%s", changes)
         return changes
 
@@ -879,15 +897,15 @@ class XpraServer(gobject.GObject, X11ServerBase):
             windowlog("cannot map window %i: not found, already removed?", wid)
             return
         assert not window.is_OR()
-        ss = self._server_sources.get(proto)
+        ss = self.get_server_source(proto)
         if ss is None:
             return
         geomlog("client %s mapped window %i - %s, at: %s", ss, wid, window, (x, y, w, h))
         self._window_mapped_at(proto, wid, window, (x, y, w, h))
         if len(packet)>=7:
             self._set_client_properties(proto, wid, window, packet[6])
-        #if not self.ui_driver:
-        #    self.set_ui_driver(ss)
+        if not self.ui_driver:
+            self.set_ui_driver(ss)
         if self.ui_driver==ss.uuid or not self._desktop_manager.is_shown(window):
             if len(packet)>=8:
                 self._set_window_state(proto, wid, window, packet[7])
@@ -904,7 +922,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
             log("cannot unmap window %i: not found, already removed?", wid)
             return
         assert not window.is_OR()
-        ss = self._server_sources.get(proto)
+        ss = self.get_server_source(proto)
         if ss is None:
             return
         self._window_mapped_at(proto, wid, window)
@@ -935,7 +953,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
             x = max(0, min(x, rw-w))
             y = max(0, min(y, rh-h))
             #tell this client to honour the new location
-            ss = self._server_sources.get(proto)
+            ss = self.get_server_source(proto)
             if ss:
                 resize_counter = max(resize_counter, self._desktop_manager.get_resize_counter(window, 1))
                 ss.move_resize_window(wid, window, x, y, w, h, resize_counter)
@@ -947,7 +965,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
         if not window:
             geomlog("cannot configure window %i: not found, already removed?", wid)
             return
-        ss = self._server_sources.get(proto)
+        ss = self.get_server_source(proto)
         if not ss:
             return
         #some "configure-window" packets are only meant for metadata updates:
@@ -959,11 +977,11 @@ class XpraServer(gobject.GObject, X11ServerBase):
             if cprops:
                 metadatalog("window client properties updates: %s", cprops)
                 self._set_client_properties(proto, wid, window, cprops)
-        #if not self.ui_driver:
-        #    self.set_ui_driver(ss)
+        if not self.ui_driver:
+            self.set_ui_driver(ss)
         is_ui_driver = self.ui_driver==ss.uuid
         shown = self._desktop_manager.is_shown(window)
-        if window.is_OR() or window.is_tray() or skip_geometry:
+        if window.is_OR() or window.is_tray() or skip_geometry or self.readonly:
             size_changed = False
         else:
             oww, owh = self._desktop_manager.window_size(window)
@@ -974,13 +992,18 @@ class XpraServer(gobject.GObject, X11ServerBase):
                 pwid = packet[10]
                 pointer = packet[11]
                 modifiers = packet[12]
+                if pwid==wid and window.is_OR():
+                    #some clients may send the OR window wid
+                    #this causes focus issues (see #1999)
+                    pwid = -1
+                mouselog("configure pointer data: %s", (pwid, pointer, modifiers))
                 if self._process_mouse_common(proto, pwid, pointer):
                     #only update modifiers if the window is in focus:
                     if self._has_focus==wid:
                         self._update_modifiers(proto, wid, modifiers)
             if window.is_tray():
                 assert self._tray
-                if not skip_geometry:
+                if not skip_geometry and not self.readonly:
                     traylog("tray %s configured to: %s", window, (x, y, w, h))
                     self._tray.move_resize(window, x, y, w, h)
                     damage = True
@@ -989,7 +1012,8 @@ class XpraServer(gobject.GObject, X11ServerBase):
                 self.last_client_configure_event = monotonic_time()
                 if is_ui_driver and len(packet)>=9:
                     changes = self._set_window_state(proto, wid, window, packet[8])
-                    damage |= len(changes)>0
+                    if changes:
+                        damage = True
                 if not skip_geometry:
                     owx, owy, oww, owh = self._desktop_manager.window_geometry(window)
                     resize_counter = 0
@@ -1064,7 +1088,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
     """ override so we can raise the window under the cursor
         (gtk raise does not change window stacking, just focus) """
     def _move_pointer(self, wid, pos, *args):
-        if wid>0 and self.last_raised!=wid:
+        if wid>0 and (self.last_raised!=wid or ALWAYS_RAISE_WINDOW):
             window = self._lookup_window(wid)
             if not window:
                 mouselog("_move_pointer(%s, %s) invalid window id", wid, pos)
@@ -1285,28 +1309,6 @@ class XpraServer(gobject.GObject, X11ServerBase):
         return False
 
 
-    def _handle_menu_rpc(self, ss, rpcid, action_type, wid, action, state, pdata, *extra):
-        if self.readonly:
-            return
-        menulog("_handle_menu_rpc%s", (ss, rpcid, action_type, wid, action, state, pdata, extra))
-        def native(args):
-            return [self.dbus_helper.dbus_to_native(x) for x in (args or [])]
-        def ok_back(*args):
-            menulog("menu rpc: ok_back%s", args)
-            ss.rpc_reply("menu", rpcid, True, native(args))
-        def err_back(*args):
-            menulog("menu rpc: err_back%s", args)
-            ss.rpc_reply("menu", rpcid, False, native(args))
-        try:
-            window = self._lookup_window(wid)
-            assert window, "window %i not found" % wid
-            window.activate_menu(action_type, action, state, pdata)
-            ok_back("done")
-        except Exception as e:
-            menulog.error("Error: cannot handle menu action %s:", action)
-            menulog.error(" %s", e)
-            err_back(str(e))
-
     def do_make_screenshot_packet(self):
         log("grabbing screenshot")
         regions = []
@@ -1324,7 +1326,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
             try:
                 with xsync:
                     img = window.get_image(0, 0, w, h)
-            except:
+            except XError:
                 log.warn("screenshot: window %s could not be captured", wid)
                 continue
             if img is None:
