@@ -1,6 +1,6 @@
 /*
  * This file is part of Xpra.
- * Copyright (C) 2016-2018 Antoine Martin <antoine@devloop.org.uk>
+ * Copyright (C) 2016-2018 Antoine Martin <antoine@xpra.org>
  * Copyright (c) 2016 Spikes, Inc.
  * Licensed under MPL 2.1, see:
  * http://www.mozilla.org/MPL/2.1/
@@ -10,8 +10,11 @@
 'use strict';
 
 var Utilities = {
-	VERSION	: "2.4.1",
+	VERSION	: "3.0.5",
 
+	exc : function() {
+		console.error.apply(console, arguments);
+	},
 	error : function() {
 		console.error.apply(console, arguments);
 	},
@@ -21,6 +24,26 @@ var Utilities = {
 	log : function() {
 		console.log.apply(console, arguments);
 	},
+	debug : function() {
+		console.debug.apply(console, arguments);
+	},
+
+	//these versions should not be redirected:
+	cexc : function() {
+		console.error.apply(console, arguments);
+	},
+	cerror : function() {
+		console.error.apply(console, arguments);
+	},
+	cwarn : function() {
+		console.log.apply(console, arguments);
+	},
+	clog : function() {
+		console.log.apply(console, arguments);
+	},
+	cdebug : function() {
+		console.debug.apply(console, arguments);
+	},
 
 	getHexUUID: function() {
 		var s = [];
@@ -59,6 +82,13 @@ var Utilities = {
 		return result;
 	},
 
+	trimString: function(str, trimLength){
+		var trimString = str.length > trimLength ?
+                    str.substring(0, trimLength - 3) + "..." :
+                    str;
+		return trimString;
+	},
+
 	getPlatformProcessor: function() {
 		//mozilla property:
 		if (navigator.oscpu){
@@ -136,6 +166,10 @@ var Utilities = {
 		if (v==null) {
 			return "us";
 		}
+		var layout = LANGUAGE_TO_LAYOUT[v];
+		if (layout) {
+			return layout;
+		}
 		//ie: v="en_GB";
 		v = v.split(',')[0];
 		var l = v.split('-', 2);
@@ -146,7 +180,7 @@ var Utilities = {
 			return '';
 		}
 		//ie: "gb"
-		var layout=l[1].toLowerCase();
+		layout=l[1].toLowerCase();
 		console.debug("getKeyboardLayout()=", layout);
 		return layout;
 	},
@@ -201,6 +235,9 @@ var Utilities = {
 		var ua = navigator.userAgent.toLowerCase();
 		return ua.includes("safari") && !ua.includes('chrome');
 	},
+	isEdge : function() {
+		return navigator.userAgent.includes("Edge");
+	},
 	isChrome : function () {
 		var isChromium = window.chrome,
 			winNav = window.navigator,
@@ -219,7 +256,27 @@ var Utilities = {
 		  }
 	},
 	isIE : function() {
-		return navigator.userAgent.includes("MSIE");
+		return navigator.userAgent.includes("MSIE") || navigator.userAgent.includes("Trident/");
+	},
+
+	is_64bit : function() {
+		var _to_check = [] ;
+		if (window.navigator.cpuClass)
+			_to_check.push((window.navigator.cpuClass + "").toLowerCase());
+		if (window.navigator.platform)
+			_to_check.push((window.navigator.platform + "").toLowerCase());
+		if (navigator.userAgent)
+			_to_check.push((navigator.userAgent + "").toLowerCase());
+		var _64bits_signatures = ["x86_64", "x86-64", "Win64", "x64;", "amd64", "AMD64", "WOW64", "x64_64", "ia64", "sparc64", "ppc64", "IRIX64"];
+		var _i, _c;
+		for (_c=0; _c<_to_check.length; _c++) {
+			for (_i=0 ; _i<_64bits_signatures.length; _i++) {
+				if (_to_check[_c].indexOf(_64bits_signatures[_i].toLowerCase())!=-1) {
+					return true;
+				}
+			}
+		}
+		return false;
 	},
 
 	getSimpleUserAgentString : function() {
@@ -386,6 +443,20 @@ var Utilities = {
 		return window.btoa(s);
 	},
 
+	convertDataURIToBinary : function (dataURI) {
+		var BASE64_MARKER = ';base64,';
+		var base64Index = dataURI.indexOf(BASE64_MARKER) + BASE64_MARKER.length;
+		var base64 = dataURI.substring(base64Index);
+		var raw = window.atob(base64);
+		var rawLength = raw.length;
+		var array = new Uint8Array(new ArrayBuffer(rawLength));
+
+		for(i = 0; i < rawLength; i++) {
+			array[i] = raw.charCodeAt(i);
+		}
+		return array;
+	},
+
 	/**
 	 * XmlHttpRequest's getAllResponseHeaders() method returns a string of response
 	 * headers according to the format described here:
@@ -433,8 +504,13 @@ var Utilities = {
 			};
 		}
 		var value = getParameter(prop);
-		if (value === undefined && typeof(sessionStorage) !== undefined) {
-			value = sessionStorage.getItem(prop);
+		try {
+			if (value === undefined && typeof(sessionStorage) !== undefined) {
+				value = sessionStorage.getItem(prop);
+			}
+		}
+		catch (e) {
+			value = null;
 		}
 		return value;
 	},
@@ -445,9 +521,25 @@ var Utilities = {
 		if(v===null) {
 			return default_value;
 		}
-		return ["true", "on", "1", "yes", "enabled"].includes(String(v).toLowerCase());
+		return ["true", "on", "1", "yes", "enabled"].indexOf(String(v).toLowerCase())!==-1;
 	},
 
+	hasSessionStorage : function() {
+		if (typeof(Storage) === "undefined") {
+			return false;
+		}
+		try {
+			var key = "just for testing sessionStorage support";
+		    sessionStorage.setItem(key, "store-whatever");
+		    sessionStorage.removeItem(key);
+		    return true;
+		}
+		catch (e) {
+			return false;
+		}
+	},
+
+
 	getConnectionInfo : function() {
 		var c = navigator.connection;
 		if (!c) {
@@ -510,3 +602,8 @@ var MOVERESIZE_DIRECTION_JS_NAME = {
         6	: "sw",
         7	: "w",
         };
+
+//convert a language code into an X11 keyboard layout code:
+var LANGUAGE_TO_LAYOUT = {
+		"da"	: "dk",
+}
\ No newline at end of file
